/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __2d_h__
#define __2d_h__

#include <GL/gl.h>

#include "world.h"

/* init the 2D system */
void init_2d(World *w);

/* free the 2D system relative stuff */
void free_2d(World *w);

/* Regenerate the box texture */
void regen_boxs(World *w);

/* Update the properties of the box texture */
void update_boxs(World *w);

/* Regenerate the font texture */
void regen_fonts(World *w, const char *fname);

/* Update the properties of the font texture */
void update_fonts(World *w);

/* switch to 2D mode ((0,0)-(100,100)) */
void activate_2d(void);

/* return to 3D mode */
void deactivate_2d(void);

/* draw a 2D box from x,y, with w,h for size */
/* states are: 0: normal,  1: selected, 2: non selectable, 3: pressed */
void draw_2d_box(float x, float y, float w, float h, int state);

/* draw a letter with its index (2D pos and zoom must be set before) */
void draw_2d_letter(int index, float x, float y, float w, float h, int sel);

/* find letter index */
/* takes the address of the char because it is planed
   to extend the text with control sequences, in order to
   access to additionnal characters or functions */
int get_letter_index(char *txt, int *shift);

/* return the size of a letter (in 0-100 units) */
float get_2d_letter_size(int size);

/* return the height of a letter (in 0-100 units) */
float get_2d_letter_height(int size);

/* return the extention of a letter (in 0-100 units) */
float get_2d_letter_extention(int index, int size);

/* return the size of a text (in 0-100 units) */
float get_2d_text_size(char *text, int size);

/* return the size of a text (in 0-100 units) */
float get_2d_text_size_full(char *text, int size, float x);

/* draw a text on the screen */
/* all text-drawing function return the lenght of the string (in 2D coord) */
float draw_2d_text(char *text, float x, float y, int size, int sel);

/* TRUE if mouse cursor was in the last text drawn */
int cursor_inside_text(void);

/* TRUE if coords given are inside the text */
int is_inside_text(char *text, float x, float y, int size, float px, float py);

/* TRUE if coords given are inside the text (centered) */
int is_inside_text_center(char *text, float x, float w, float y, int size, float px, float py);

/* TRUE if coords given are inside the text (right justified) */
int is_inside_text_right(char *text, float x, float w, float y, int size, float px, float py);

/* draw a text on the screen, centered betwwen (x) and (x+w) */
/* WARNING: dont use TABs with centered text: position will be erroneous */
float draw_2d_text_center(char *text, float x, float w, float y, int size, int sel);

/* draw a text on the screen, right-aligned betwwen (x) and (x+w) */
/* WARNING: dont use TABs with centered text: position will be erroneous */
float draw_2d_text_right(char *text, float x, float w, float y, int size, int sel);

/* set transparency for drawing operations */
void set_2d_transparency(GLfloat transp);

#endif /* __2d_h__ */
