/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __explosion_h__
#define __explosion_h__

typedef struct _Blast Blast;

#include "world.h"
#include "player.h"

/* The fireball from an explosion */
struct _Blast
{
  World *world;
  Game *game;
  Round *round;

  float pos[3];
  float age;
  float speed, radius, current_radius;
  float color[4];

  Blast *last, *next;
};

/* Causes an explosion at 'pos', with radius 'radius', and force 'force'*/
void calc_explosion(Player *shooter, float *pos, float radius, float force);

/* Remove a blast from the list */
void remove_blast(Blast *blast);

/* Updates all the blasts in the world */
void update_blasts(Round *round, float dt);

/* Draws all the blasts in the world */
void draw_blasts(Round *round);

/* Remove all blasts */
int clear_blasts(Round *round);

#endif /* __explosion_h__ */
