/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __player_stats__
#define __player_stats__

typedef struct _PlayerStat PlayerStat;

#include "player.h"

struct _PlayerStat
{
  Player *player;

  /* Permament statistics */
  int kills, losses;
  float damage_taken, damage_inflicted;

  /* Statistics that may be calculated (i.e. for the AI) */
  float range;
  int visible, closest, strongest, weakest;

  /* A value that assess' the threat of the player,
     and how easy a target this is */
  float threat, easiness;

  PlayerStat *last, *next;
};

/* Build a list of statistics on opponents for all the players */
void build_all_player_statistics(World *world);

/* Build a list of statistics on opponents for a player */
void build_player_statistics(Player *player);

/* Record damage to victim in players statistics */
void record_damage(Player *player, Player *victim, float damage);

/* Record a kill in players statistics */
void record_kill(Player *killer, Player *victim);

/* Clear out the statistics the player stores on opponents */
void clear_player_stats(Player *player);

/* Remove all statistics that a player stores on opponents */
void free_player_stats(Player *player);

#endif /* __player_stats__ */
