/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* round_config.c
   Config game rounds */

#include "round_config.h"
#include "round.h"
#include "glut_interface.h"
#include "display_stack.h"
#include "2d.h"

static int selected = 0;

/* Reset the round config */
int reset_round_config(void)
{
  selected = 0;

  return TRUE;
}

/* Draw a menu when in a round */
void draw_round_menu(Round *round)
{
  /* draw messages ... */
  activate_2d();

  set_2d_transparency(1.);
  draw_2d_box(1, 1, 98, 98, 0);

  draw_2d_text_center("Up/Down: choose, Ret: select, Esc: cancel",
                      1, 98, 2.1, -1, 0);

  draw_2d_text_center("Quit round", 1, 98, 60, 0, selected == 0);
  if(cursor_inside_text())
    selected = 0;
  draw_2d_text_center("Key config", 1, 98, 50, 0, selected == 1);
  if(cursor_inside_text())
    selected = 1;
  draw_2d_text_center("Video Config", 1, 98, 40, 0, selected == 2);
  if(cursor_inside_text())
    selected = 2;
  draw_2d_text_center("Continue", 1, 98, 30, 0, selected == 3);
  if(cursor_inside_text())
    selected = 3;

  deactivate_2d();
}

/* Manage the menu used in a round */
void manage_round_menu_input(Round *round, int key)
{
  World *world;

  world = round->world;

  if(key == KEY_DOWN)
  {
    selected++;
    if(selected >= 4)
      selected = 0;
  }

  else if(key == KEY_UP)
  {
    selected--;
    if(selected < 0)
      selected = 3;
  }

  else if(key == KEY_RETURN || key == KEY_MOUSE1)
  {
    switch(selected)
    {
    case 0: /* quit */
      reset_round_config();
      end_round(round);
      break;
    case 1: /* key config */
      push_display_mode(world, NATOW_DISPLAY_KEY_CONFIG);
      break;
    case 2: /* video config */
      push_display_mode(world, NATOW_DISPLAY_VIDEO_CONFIG);
      break;
    case 3: /* continue */
      pop_display_mode(world);
      break;
    }
  }

  else if(key == KEY_ESC || key == KEY_MOUSE3)
    pop_display_mode(world);
}
