/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* manage satellites creation, drawing and updating */

#ifndef __satellite_h__
#define __satellite_h__

typedef struct _Satellite Satellite;

#include "world.h"

/* Satellites */
struct _Satellite
{
  World *world;

  /* global info */
  float radius;
  GLfloat color[4]; /* satellite color */
  float ax, bx;
  float ay, by;
  float az, bz;
  float current;
  /* current state */
  float px, py, pz;  /* current position */
  float rotx, roty, rotz; /* current rotations */
};

/* create a new satellite */
Satellite *create_satellite(World *w, char *params);

/* delete a satellite */
void delete_satellite(Satellite *sat);

/* draw a given satellite */
void draw_satellite(Satellite *sat);

/* update position of a given satellite */
void update_satellite(Satellite *sat, float dt);

/* Free a sattelite */
int free_satellite(Satellite *satellite);

#endif /* __satellite_h__ */
