/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* shade.c
   This file contains the routines for drawing a shade between
   different game screens (fade to black) */

#include <stdio.h>
#include <GL/gl.h>
#include <string.h>

#include "shade.h"
#include "world.h"
#include "text.h"
#include "2d.h"
#include "display_stack.h"
#include "glut_interface.h"


/* internal structure to store background data */
GLvoid *current_background = NULL;


/* init shading */
int init_shading(World* world)
{
  /* shading desactivated */
  if (world->is_shading == FALSE)
    {/* dont start shade */
    return(FALSE);
    }

  if (current_background == NULL)
    {
    current_background = malloc(sizeof(unsigned char)*4*
                                 world->size_x*world->size_y);
    }
  if (current_background == NULL)
    return(FALSE);

  /* store the current background */
  glReadPixels(0, 0, world->size_x, world->size_y,
               GL_RGBA, GL_UNSIGNED_BYTE,
               current_background);

  return(TRUE);
}

/* free the shading system */
void free_shading(World *world)
{
  if (current_background != NULL)
    free(current_background);
  current_background = NULL;
}



/* draw a shade-to-black over the current screen */
void draw_shade(World*world)
{
  activate_2d();

  glPushAttrib(GL_ALL_ATTRIB_BITS);
  glDisable(GL_TEXTURE);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glDisable(GL_LIGHTING);

  if (world->shade_mode == NATOW_SHADE_DOWN)
    glDrawPixels(world->size_x, world->size_y,
                 GL_RGBA, GL_UNSIGNED_BYTE,
                 (const GLvoid *)current_background);

  glColor4f(0., 0., 0., world->shade_level);

  glBegin(GL_QUADS);
  
  glVertex2f(0,   0);
  glVertex2f(100, 0);
  glVertex2f(100, 100);
  glVertex2f(0,   100);
  
  glEnd();

  glPopAttrib();

  deactivate_2d();
}

/* update the shade state (returns TRUE when finished) */
int update_shade_down(World*world, float dt)
{
  world->shade_level += 1.2 * dt;

  if (world->shade_level > 1.)
    {
    world->shade_level = 1.;
    return(TRUE);
    }
  return(FALSE);
}

/* update the shade state  (returns TRUE when finished) */
int update_shade_up(World*world, float dt)
{
  world->shade_level -= 1.3 * dt;

  if (world->shade_level < 0.)
    {
    world->shade_level = 0.;
    return(TRUE);
    }
  return(FALSE);
}


