/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* tanks.c
   This file contains the "hard-wired" units at present - in the future they
   will be loaded from seperate files using GLASS see glass.sourceforge.net
   for details */

#include <math.h>
#include <GL/gl.h>

#include "tanks.h"
#include "world.h"

void generate_tank_lists(World *world)
{
  /* Make the display lists */
  world->lists.shell = glGenLists(1);
  world->lists.missile = glGenLists(1);
  world->lists.bomblet = glGenLists(1);
  world->lists.parachute = glGenLists(1);
#ifndef USE_GLASS
  world->lists.body = glGenLists(1);
  world->lists.turret = glGenLists(1);
  world->lists.gun = glGenLists(1);
#endif

  /* Shell */
  glNewList(world->lists.shell, GL_COMPILE);
    draw_shell();
  glEndList();

  /* Missile */
  glNewList(world->lists.missile, GL_COMPILE);
    draw_missile();
  glEndList();

  /* Cluster bomb */
  glNewList(world->lists.bomblet, GL_COMPILE);
    draw_cbomb();
  glEndList();

  /* Parachute */
  glNewList(world->lists.parachute, GL_COMPILE);
    draw_parachute();
  glEndList();

#ifndef USE_GLASS
  /* Unit body */
  glNewList(world->lists.body, GL_COMPILE);
    draw_tank1_body();
  glEndList();

  /* Unit turret */
  glNewList(world->lists.turret, GL_COMPILE);
    draw_tank1_turret();
  glEndList();

  /* Unit gun */
  glNewList(world->lists.gun, GL_COMPILE);
    draw_tank1_gun();
  glEndList();
#endif
}

void draw_shell(void)
{
  float c[3], theta, sin_theta, cos_theta;
  float len = sqrt(0.26f);

  glColor3f(1.0, 0.0, 0.0);
  c[0] = 1.0;
  c[1] = 0.0;
  c[2] = 0.0;
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, c);

  /* Nose */
  glBegin(GL_TRIANGLE_FAN);

  /* Tip */
  glNormal3f(0.0, 0.0, 1.0);
  glVertex3f(0.0, 0.0, 0.5);

  /* Cone */
  for(theta = 0; theta <= 360.0; theta += 60.0)
  {
    sin_theta = sin((M_PI / 180.0) * theta);
    cos_theta = cos((M_PI / 180.0) * theta);

    glNormal3f(0.5f * sin_theta / len, 0.5f * cos_theta / len, 0.1 / len);
    glVertex3f(0.1 * cos_theta, 0.1 * sin_theta, 0.0);
  }

  glEnd();

  /* Bulk */
  glBegin(GL_TRIANGLE_STRIP);

  for(theta = 0; theta <= 360.0; theta += 60.0)
  {
    sin_theta = sin((M_PI / 180.0) * theta);
    cos_theta = cos((M_PI / 180.0) * theta);

    glNormal3f(sin_theta, cos_theta, 0.0);
    glVertex3f(0.1 * cos_theta, 0.1 * sin_theta, 0.0);
    glVertex3f(0.1 * cos_theta, 0.1 * sin_theta, -0.16666667);
  }

  glEnd();

  /* End */
  glNormal3f(0.0f, 0.0f, -1.0f);
  glBegin(GL_TRIANGLE_FAN);

  glVertex3f(0.0f, 0.0f, -0.16666667f);

  for(theta = 0.0f; theta <= 360.0f; theta += 60.0)
  {
    sin_theta = sin((M_PI / 180.0) * theta);
    cos_theta = cos((M_PI / 180.0) * theta);

    glVertex3f(0.1f * cos_theta, -0.1f * sin_theta, -0.16666667);
  }

  glEnd();
}

void draw_missile(void)
{
  float theta, sin_theta, cos_theta, len = sqrt(0.010625f);

  /* tip */
  glBegin(GL_TRIANGLE_FAN);
  glNormal3f(0.0f, 0.0f, 1.0f);
  glVertex3f(0.0f, 0.0f, 0.6f);
  for(theta = 0.0f; theta < 2.0f * M_PI; theta += M_PI / 3.0f)
  {
    sin_theta = sin(theta);
    cos_theta = cos(theta);

    glNormal3f(0.1f * sin_theta / len, 0.1f * cos_theta / len, 0.025f / len);
    glVertex3f(0.025f * cos_theta, 0.025f * sin_theta, 0.5f);
  }
  glEnd();

  /* Shaft */
  glBegin(GL_QUAD_STRIP);
  for(theta = 0.0f; theta < 2.0f * M_PI; theta += M_PI / 3.0f)
  {
    sin_theta = sin(theta);
    cos_theta = cos(theta);

    glNormal3f(sin_theta, cos_theta, 0.0f);
    glVertex3f(0.025f * cos_theta, 0.025f * sin_theta, 0.5f);
    glVertex3f(0.025f * cos_theta, 0.025f * sin_theta, -0.5f);
  }
  glEnd();

  /* End */
  glBegin(GL_TRIANGLE_FAN);
  glNormal3f(0.0f, 0.0f, -1.0f);
  glVertex3f(0.0f, 0.0f, -0.5f);
  for(theta = 0.0f; theta < 2.0f * M_PI; theta += M_PI / 3.0f)
    glVertex3f(0.025f * cos(theta), -0.025f * sin(theta), -0.5f);
  glEnd();

  /* Fins */
  glBegin(GL_TRIANGLES);
  /* At 120deg */
  for(theta = 0.0f; theta < 2.0f * M_PI; theta += 2.0f * M_PI / 3.0f)
  {
    sin_theta = sin(theta);
    cos_theta = cos(theta);

    glNormal3f(-cos_theta, sin_theta, 0.0f);

    /* Front */
    glVertex3f(0.0f, 0.0f, 0.3f);
    glVertex3f(0.0f, 0.0f, 0.5f);
    glVertex3f(0.07f * cos_theta, 0.07f * sin_theta, 0.3f);

    /* Rear */
    glVertex3f(0.0f, 0.0f, -0.45f);
    glVertex3f(0.0f, 0.0f, -0.2f);
    glVertex3f(0.1f * cos_theta, 0.1f * sin_theta, -0.35f);
    glVertex3f(0.0f, 0.0f, -0.45f);
    glVertex3f(0.1f * cos_theta, 0.1f * sin_theta, -0.35f);
    glVertex3f(0.1f * cos_theta, 0.1f * sin_theta, -0.45f);

    glNormal3f(-cos_theta, sin_theta, 0.0f);

    /* Front */
    glVertex3f(0.0f, 0.0f, 0.3f);
    glVertex3f(0.07f * cos_theta, 0.07f * sin_theta, 0.3f);
    glVertex3f(0.0f, 0.0f, 0.5f);

    /* Rear */
    glVertex3f(0.0f, 0.0f, -0.45f);
    glVertex3f(0.1f * cos_theta, 0.1f * sin_theta, -0.35f);
    glVertex3f(0.0f, 0.0f, -0.2f);
    glVertex3f(0.0f, 0.0f, -0.45f);
    glVertex3f(0.1f * cos_theta, 0.1f * sin_theta, -0.45f);
    glVertex3f(0.1f * cos_theta, 0.1f * sin_theta, -0.35f);
  }
  glEnd();
}

void draw_cbomb(void)
{
  float theta, sin_theta, cos_theta, len = sqrt(0.565f);

  /* Top */
  glBegin(GL_TRIANGLE_FAN);
  glNormal3f(0.0f, 0.0f, 1.0f);
  glVertex3f(0.0f, 0.0f, 0.075f);
  for(theta = 0.0f; theta < 2.0f * M_PI; theta += M_PI / 3.0f)
  {
    sin_theta = sin(theta);
    cos_theta = cos(theta);

    glNormal3f(0.075f * sin_theta / len, 0.075f * cos_theta / len, 0.05f / len);
    glVertex3f(0.05f * cos_theta, 0.05f * sin_theta, 0.0f);
  }
  glEnd();

  /* Bottom */
  glBegin(GL_TRIANGLE_FAN);
  glNormal3f(0.0f, 0.0f, -1.0f);
  glVertex3f(0.0f, 0.0f, -0.075f);
  for(theta = 0.0f; theta < 2.0f * M_PI; theta += M_PI / 3.0f)
  {
    sin_theta = sin(theta);
    cos_theta = cos(theta);

    glNormal3f(-0.075f * sin_theta / len, 0.075f * cos_theta / len, -0.05f / len);
    glVertex3f(0.05f * cos_theta, -0.05f * sin_theta, 0.0f);
  }
  glEnd();
}

void draw_parachute(void)
{
  float c[4] = {0.9f, 0.9f, 0.9f, 0.9f};
  float theta, sin_theta, cos_theta, len = sqrt(0.1125f);

  /* Draw the parachute */
  glColor4fv(c);
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, c);

  /* Top */
  glBegin(GL_TRIANGLE_FAN);
  glNormal3f(0.0f, 0.0f, -1.0f);
  glVertex3f(0.0f, 0.0f, -0.65f);
  for(theta = 0.0f; theta < 2.0f * M_PI; theta += M_PI / 3.0f)
  {
    sin_theta = sin(theta);
    cos_theta = cos(theta);

    glNormal3f(0.15f * -sin_theta / len, 0.15f * cos_theta / len, -0.3f / len);
    glVertex3f(0.3f * cos_theta, 0.3f * -sin_theta, -0.5f);
  }
  glEnd();

  /* Bottom */
  glBegin(GL_TRIANGLE_FAN);
  glNormal3f(0.0f, 0.0f, 1.0f);
  glVertex3f(0.0f, 0.0f, -0.65f);
  for(theta = 0.0f; theta < 2.0f * M_PI; theta += M_PI / 3.0f)
  {
    sin_theta = sin(theta);
    cos_theta = cos(theta);

    glNormal3f(0.15f * sin_theta / len, 0.15f * cos_theta / len, 0.3f / len);
    glVertex3f(0.3f * cos_theta, 0.3f * sin_theta, -0.5f);
  }
  glEnd();

  glBegin(GL_LINES);
  for(theta = 0.0f; theta < 2.0f * M_PI; theta += 2.0 * M_PI / 3.0f)
  {
    sin_theta = sin(theta);
    cos_theta = cos(theta);

    glVertex3f(0.0f, 0.0f, 0.0f);
    glVertex3f(0.3f * cos_theta, 0.3f * sin_theta, -0.5f);
  }
  glEnd();
}

void draw_tank1_body(void)
{
  draw_track1(-2.0, 0.5);
  draw_track1(1.5, 0.5);
  draw_chassis1(3.0);
}

void draw_tank1_turret(void)
{
  draw_turret1(0.0, 0.0);
}

void draw_tank1_gun(void)
{
  draw_gun1();
}

void draw_track1(float x1, float w)
{
  float x2 = x1 + w;
  float nx;

  if(w > 0)
    nx = 1.0;
  else
    nx = -1.0;

  glBegin(GL_QUADS);

  /* tracks */
  glNormal3f(0.0, 0.89442719, 0.4472136);
  glTexCoord2f(0,0);
  glVertex3f(x2, 0.0, 1.0);
  glTexCoord2f(0,1);
  glVertex3f(x2, 0.5, 2.0);
  glTexCoord2f(1,1);
  glVertex3f(x1, 0.5, 2.0);
  glTexCoord2f(1,0);
  glVertex3f(x1, 0.0, 1.0);

  glNormal3f(0.0, 0.0, 1.0);
  glTexCoord2f(0,0);
  glVertex3f(x2, 0.5, 2.0);
  glTexCoord2f(0,1);
  glVertex3f(x2, 1.0, 2.0);
  glTexCoord2f(1,1);
  glVertex3f(x1, 1.0, 2.0);
  glTexCoord2f(1,0);
  glVertex3f(x1, 0.5, 2.0);

  glNormal3f(0.0, 0.70710678, 0.70710678);
  glTexCoord2f(0,0);
  glVertex3f(x2, 1.0, 2.0);
  glTexCoord2f(0,1);
  glVertex3f(x2, 1.5, 1.5);
  glTexCoord2f(1,1);
  glVertex3f(x1, 1.5, 1.5);
  glTexCoord2f(1,0);
  glVertex3f(x1, 1.0, 2.0);

  glNormal3f(0.0, 1.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(x2, 1.5, 1.5);
  glTexCoord2f(0,1);
  glVertex3f(x2, 1.5, -1.0);
  glTexCoord2f(1,1);
  glVertex3f(x1, 1.5, -1.0);
  glTexCoord2f(1,0);
  glVertex3f(x1, 1.5, 1.5);

  glNormal3f(0.0, 0.89442719, -0.4472136);
  glTexCoord2f(0,0);
  glVertex3f(x2, 1.5, -1.0);
  glTexCoord2f(0,1);
  glVertex3f(x2, 1.0, -2.0);
  glTexCoord2f(1,1);
  glVertex3f(x1, 1.0, -2.0);
  glTexCoord2f(1,0);
  glVertex3f(x1, 1.5, -1.0);

  glNormal3f(0.0, 0.0, -1.0);
  glTexCoord2f(0,0);
  glVertex3f(x2, 1.0, -2.0);
  glTexCoord2f(0,1);
  glVertex3f(x2, 0.5, -2.0);
  glTexCoord2f(1,1);
  glVertex3f(x1, 0.5, -2.0);
  glTexCoord2f(1,0);
  glVertex3f(x1, 1.0, -2.0);

  glNormal3f(0.0, -0.70710678, -0.70710678);
  glTexCoord2f(0,0);
  glVertex3f(x2, 0.5, -2.0);
  glTexCoord2f(0,1);
  glVertex3f(x2, 0.0, -1.5);
  glTexCoord2f(1,1);
  glVertex3f(x1, 0.0, -1.5);
  glTexCoord2f(1,0);
  glVertex3f(x1, 0.5, -2.0);

  glNormal3f(0.0, -1.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(x2, 0.0, -1.5);
  glTexCoord2f(0,1);
  glVertex3f(x2, 0.0, 1.0);
  glTexCoord2f(1,1);
  glVertex3f(x1, 0.0, 1.0);
  glTexCoord2f(1,0);
  glVertex3f(x1, 0.0, -1.5);

  glEnd();

  /* Draw sides */
  glNormal3f(-nx, 0.0, 0.0);
  glBegin(GL_TRIANGLE_STRIP);

  glVertex3f(x1, 0.5, 2.0);
  glVertex3f(x1, 1.0, 2.0);
  glVertex3f(x1, 0.0, 1.0);
  glVertex3f(x1, 1.5, 1.5);
  glVertex3f(x1, 0.0, -1.5);
  glVertex3f(x1, 1.5, -1.0);
  glVertex3f(x1, 0.5, -2.0);
  glVertex3f(x1, 1.0, -2.0);

  glEnd();

  glNormal3f(nx, 0.0, 0.0);
  glBegin(GL_TRIANGLE_STRIP);

  glVertex3f(x2, 0.5, 2.0);
  glVertex3f(x2, 0.0, 1.0);
  glVertex3f(x2, 1.0, 2.0);
  glVertex3f(x2, 0.0, -1.5);
  glVertex3f(x2, 1.5, 1.5);
  glVertex3f(x2, 0.5, -2.0);
  glVertex3f(x2, 1.5, -1.0);
  glVertex3f(x2, 1.0, -2.0);

  glEnd();
}

void draw_chassis1(float w)
{
  w /= 2;

  /* edge */
  glBegin(GL_QUADS);

  glNormal3f(0.0, -0.70710678, 0.70710678);
  glTexCoord2f(0,0);
  glVertex3f(w, 0.5, 1.0);
  glTexCoord2f(0,.5);
  glVertex3f(w, 1.0, 1.5);
  glTexCoord2f(1,.5);
  glVertex3f(-w, 1.0, 1.5);
  glTexCoord2f(1,0);
  glVertex3f(-w, 0.5, 1.0);

  glNormal3f(0.0, 0.0, 1.0);
  glTexCoord2f(0,0);
  glVertex3f(w, 1.0, 1.5);
  glTexCoord2f(0,.5);
  glVertex3f(w, 1.5, 1.5);
  glTexCoord2f(1,.5);
  glVertex3f(-w, 1.5, 1.5);
  glTexCoord2f(1,0);
  glVertex3f(-w, 1.0, 1.5);

  glNormal3f(0.0, 1.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(w, 1.5, 1.5);
  glTexCoord2f(0,.5);
  glVertex3f(w, 1.5, -1.5);
  glTexCoord2f(1,.5);
  glVertex3f(-w, 1.5, -1.5);
  glTexCoord2f(1,0);
  glVertex3f(-w, 1.5, 1.5);

  glNormal3f(0.0, 0.0, -1.0);
  glTexCoord2f(0,0);
  glVertex3f(w, 1.5, -1.5);
  glTexCoord2f(0,.5);
  glVertex3f(w, 0.5, -1.5);
  glTexCoord2f(1,.5);
  glVertex3f(-w, 0.5, -1.5);
  glTexCoord2f(1,0);
  glVertex3f(-w, 1.5, -1.5);

  glNormal3f(0.0, -1.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(w, 0.5, -1.5);
  glTexCoord2f(0,.5);
  glVertex3f(w, 0.5, 1.0);
  glTexCoord2f(1,.5);
  glVertex3f(-w, 0.5, 1.0);
  glTexCoord2f(1,0);
  glVertex3f(-w, 0.5, -1.5);

  glEnd();

  /* sides */
  glBegin(GL_TRIANGLES);

  glNormal3f(1.0, 0.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(w, 1.5, 1.5);
  glTexCoord2f(0,1);
  glVertex3f(w, 0.5, -1.5);
  glTexCoord2f(1,1);
  glVertex3f(w, 1.5, -1.5);

  glNormal3f(-1.0, 0.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(-w, 1.5, 1.5);
  glTexCoord2f(0,1);
  glVertex3f(-w, 1.5, -1.5);
  glTexCoord2f(1,1);
  glVertex3f(-w, 0.5, -1.5);

  glEnd();
}

void draw_turret1(float x, float y)
{
  /* edge */
  glBegin(GL_QUADS);

  /* back */
  glNormal3f(0.0, 0.0, -1.0);
  glTexCoord2f(0,0);
  glVertex3f(x + 0.5, 1.5, y - 0.5);
  glTexCoord2f(0,1);
  glVertex3f(x - 0.5, 1.5, y - 0.5);
  glTexCoord2f(1,1);
  glVertex3f(x - 0.5, 2.0, y - 0.5);
  glTexCoord2f(1,0);
  glVertex3f(x + 0.5, 2.0, y - 0.5);

  /* top */
  glNormal3f(0.0, 1.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(x + 0.5, 2.0, y - 0.5);
  glTexCoord2f(0,1);
  glVertex3f(x - 0.5, 2.0, y - 0.5);
  glTexCoord2f(1,1);
  glVertex3f(x - 0.5, 2.0, y + 0.5);
  glTexCoord2f(1,0);
  glVertex3f(x + 0.5, 2.0, y + 0.5);

  /* front */
  glNormal3f(0.0, 0.70710678, 0.70710678);
  glTexCoord2f(0,0);
  glVertex3f(x + 0.5, 2.0, y + 0.5);
  glTexCoord2f(0,1);
  glVertex3f(x - 0.5, 2.0, y + 0.5);
  glTexCoord2f(1,1);
  glVertex3f(x - 0.5, 1.5, y + 1.0);
  glTexCoord2f(1,0);
  glVertex3f(x + 0.5, 1.5, y + 1.0);

  /* sides */
  /* Left */
  glNormal3f(1.0, 0.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(x + 0.5, 1.5, y + 1.0);
  glTexCoord2f(1,0);
  glVertex3f(x + 0.5, 1.5, y - 0.5);
  glTexCoord2f(1,.6);
  glVertex3f(x + 0.5, 2.0, y - 0.5);
  glTexCoord2f(0,.6);
  glVertex3f(x + 0.5, 2.0, y + 0.5);

  /* Right */
  glNormal3f(-1.0, 0.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(x - 0.5, 1.5, y + 1.0);
  glTexCoord2f(1,0);
  glVertex3f(x - 0.5, 2.0, y + 0.5);
  glTexCoord2f(1,.6);
  glVertex3f(x - 0.5, 2.0, y - 0.5);
  glTexCoord2f(0,.6);
  glVertex3f(x - 0.5, 1.5, y - 0.5);

  glEnd();
}

void draw_gun1(void)
{
  /* gun */
  glBegin(GL_QUADS);

  /* bottom */
  glNormal3f(0.0, -1.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(-0.125, -0.125, 0.0);
  glTexCoord2f(0,.2);
  glVertex3f(0.125, -0.125, 0.0);
  glTexCoord2f(1,.2);
  glVertex3f(0.125, -0.125, 2.5);
  glTexCoord2f(1,0);
  glVertex3f(-0.125, -0.125, 2.5);

  /* right */
  glTexCoord2f(0,0);
  glNormal3f(1.0, 0.0, 0.0);
  glVertex3f(0.125, -0.125, 0.0);
  glTexCoord2f(0,.2);
  glVertex3f(0.125, 0.125, 0.0);
  glTexCoord2f(1,.2);
  glVertex3f(0.125, 0.125, 2.5);
  glTexCoord2f(1,0);
  glVertex3f(0.125, -0.125, 2.5);

  /* top */
  glNormal3f(0.0, 1.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(0.125, 0.125, 0.0);
  glTexCoord2f(0,.2);
  glVertex3f(-0.125, 0.125, 0.0);
  glTexCoord2f(1,.2);
  glVertex3f(-0.125, 0.125, 2.5);
  glTexCoord2f(1,0);
  glVertex3f(0.125, 0.125, 2.5);

  /* left */
  glNormal3f(-1.0, 0.0, 0.0);
  glTexCoord2f(0,0);
  glVertex3f(-0.125, 0.125, 0.0);
  glTexCoord2f(0,.2);
  glVertex3f(-0.125, -0.125, 0.0);
  glTexCoord2f(1,.2);
  glVertex3f(-0.125, -0.125, 2.5);
  glTexCoord2f(1,0);
  glVertex3f(-0.125, 0.125, 2.5);

  /* end */
  glNormal3f(0.0, 0.0, 1.0);
  glTexCoord2f(0,0);
  glVertex3f(-0.125, -0.125, 2.5);
  glTexCoord2f(.4,0);
  glVertex3f(0.125, -0.125, 2.5);
  glTexCoord2f(.4,.4);
  glVertex3f(0.125, 0.125, 2.5);
  glTexCoord2f(0,.4);
  glVertex3f(-0.125, 0.125, 2.5);

  glEnd();
}
