/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __world_h__
#define __world_h__

typedef enum   _NatowShadeMode NatowShadeMode;
typedef struct _DisplayLists DisplayLists;
typedef struct _World World;

#include "game.h"
#include "map.h"
#include "display_stack.h"
#include "camera.h"
#include "tank.h"
#include "munition.h"
#include "player.h"
#include "projectile.h"
#include "trails.h"
#include "explosion.h"
#include "smoke.h"
#include "objects.h"
#include "bird.h"
#include "satellite.h"
#include "keymap.h"

enum _NatowShadeMode
{
  NATOW_SHADE_NONE = 0,
  NATOW_SHADE_DOWN,
  NATOW_SHADE_UP,
};

/* A structure of the display lists used in NATOW
   (made obsolete by GLASS) */
struct _DisplayLists
{
  GLuint sky, skirt, lakes;
  GLuint shell, missile, bomblet;
  GLuint parachute;
  GLuint body, turret, gun;
};

#include "level_list.h"

/* The base structure -- everything links from here */
struct _World
{
  /* The game being played */
  Game *game;

  /* The total elapsed time, in seconds, from the start of n.a.t.o.w. */
  float total_elapsed_time;
  /* Same but restricted to 0-2Pi */
  float total_elapsed_time_trigo;

  /* If TRUE, give the user some help */
  int use_tutorial;

  /* The basic functions */
  void (*display_func)(World *world);
  void (*reshape_func)(World *world, int w, int h);
  void (*keyboard_func)(World *world, int key, int x, int y);
  void (*mouse_func)(World *world, int button, int state, int x, int y);
  void (*motion_func)(World *world, int x, int y, int dx, int dy);
  void (*update_func)(World *world, float dt);
  void (*move_func)(World *world, int x, int y);
  void (*entry_func)(World *world, int state);

  int cursor_inside;        /* Cursor is in the window */
  int paused_from_cursor;   /* Pause mode is due to cursor exit */

  /* The level of detail (LOD) distances */
  unsigned int LOD_boundary1, LOD_boundary2, LOD_boundary3;

  /* The resolution of the map texture */
  float tex_x_resolution, tex_y_resolution;

  /* The availiable levels */
  LevelEntry *levels;

  DisplayStack *display_stack;
  NatowShadeMode shade_mode;
  float shade_level;

  /* A list of the different types of tanks available */
  int ntanks;
  Tank *tank_list;

  /* A list of the different types of weapons available */
  int nmunitions;
  Munition *munition_list;

  /* A list of the players availiable */
  Player *player_list;

  /* satellites */
  /* TODO: Move to Game structure */
  Satellite *sun, *moon1, *moon2;

  /* low-detail flag */
  int low_detail;
  /* hi-detail flag */
  int hi_detail;

  /* flags */
  int is_smooth, is_fog, is_pin, is_grid, is_light, is_water, is_texture;
  int is_sky, is_bird;
  int is_shading;  /* shading between screens */
  int is_level_fog; /* the state of the fog */
  int smoke_percent;
  int plasma_detail;
  GLenum texture_mode;

  /* in solo mode, show the other tank health */
  int show_players_state;

  /* NOTE: temp */
  float fps;
  float max_dt;

  /* The requested screen resolution */
  int video_size_x, video_size_y;
  /* The real screen resolution */
  int size_x, size_y;
  /* If in fullscreen mode */
  int fullscreen;

  /* The openGL display lists for static objects */
  DisplayLists lists;

  /* The keymap */
  KeyEntry *key_list;
};

World *new_world(void);

/* Free everything */
int free_world(World *world);

#endif /* __world_h__ */
